#include "main.h"

#define SENZOR1 0
#define SENZOR2 6

static int krok=1;
int smer=0;

// funkcia, ktora vracia hodnotu pre danu rychlost zvoleneho kolesa (aj so smerom)
int rychlost_kolesa(char koleso, int rychlost){
	int tmp;

	// hodnoty pre jednotlive rychlosti su cisto experimentalne (0 - stop)
	switch(koleso){
    	case 'r':
		    if(smer>0) tmp=smer*krok; else tmp=0;
	    	switch(rychlost){
			 	case 1: return 2818+tmp;
		 		case 2: return 2834+tmp;
		 		case 3: return 2850+tmp;
		 		case 4: return 2872+tmp;
		 		case 5: return 2900+tmp;
		 		default:  return 2782;
			}
	 		break;
    	case 'l':
		    if(smer<0) tmp=-smer*krok; else tmp=0;
	    	switch(rychlost){
			 	case 1: return 2784-tmp;
		 		case 2: return 2766-tmp;
		 		case 3: return 2748-tmp;
		 		case 4: return 2726-tmp;
		 		case 5: return 2698-tmp;
		 		default:  return 2820;
			}		
			break;
  	}

	return 0;
}

// spustenie motorcekov
void forward(void)
{
  ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
  set_servo(1, rychlost_kolesa('r', 0));
  set_servo(4, rychlost_kolesa('l', 0));
  servo_123_on();
  servo_456_on();
  set_servo(1, rychlost_kolesa('r', 0));
  set_servo(4, rychlost_kolesa('l', 0));
}


// zastavenie motorcekov
void stop(void)
{
  servo_123_off();
  servo_456_off();
}

// nastavenie smeru vzad
void cuvni(void)
{
  set_servo(1, SERVO_FW);
  set_servo(4, SERVO_BW);
}

// vypisovanie vsetkych senzorov - nepouzite
void senzory(void)
{
  while (!new_packet)
  {
     printf("%4d %4d %4d %4d %4d %4d %4d\n", adc_vals[0], adc_vals[1], adc_vals[2], adc_vals[3], adc_vals[4], adc_vals[5], adc_vals[6]);
	 wait(50);
  }
}

void help(void)
{
	printf("\n");
	printf("+-----------------------------+\n");
	printf("| SBOT tlaciaci vozik... v0.9 |\n");
	printf("+-----------------------------+\n");
	printf("| Ovladanie:                  |\n");
	printf("| [a] - pozdravenie SBOTa     |\n");
	printf("| [h] - zobrazenie pomoci     |\n");
	printf("| [e] - zobrazovanie senzorov |\n");
	printf("| [f] - pohyb dopredu         |\n");
	printf("| [s] - zastavenie pohybu     |\n");
	printf("| [c] - pohyb dozadu          |\n");
	printf("| [+] - zrysovanie rychlosti  |\n");
	printf("| [e] - znizovanie rychlosti  |\n");
	printf("+-----------------------------+\n");
	printf("|     (c) Juraj Ladicky, 2010 |\n");
	printf("+-----------------------------+\n");
	printf("\n");
}

///////////////////////////////////////////////////////////////////////////////
int main(void)
{
    int senzory;			// indikator, ci sa maju zobrazovat senzory alebo nie
	int timer;				// casovac, aby sa senzory vypisovali iba kazdych 50 ms (obsluha je kazdych 10ms)
	int rychlost;			// zapnuta rychlost pohybu SBOTa
	int cuvanie;			// indikator, ako dlho este bude SBOT cuvat

	io_init();

	// uvodne prebliknutie lediek
	set_leds(0x01);	wait(100);
	set_leds(0x02);	wait(100);
	set_leds(0x04);	wait(100);
	set_leds(0x08);	wait(100);
	set_leds(0x00);

	get_sbot_number();
	stdout = &usart1;
	stdin = &usart1;


	help();

	// na zaciatku sa senzory nezobrazuju, casovac senzorov je 0, rychlost je 1 a necuvame
	senzory=0; timer=0; rychlost=1; cuvanie=0;

	while(1)
	{
		if(new_packet == 1)	//if we received a new packed ,we should take a look at it. 
		{
			new_packet = 0;
			// obsluha stlacenych klaves
			switch (newchar)
			{
			case 'a': printf("Hello...\n");
			          break;
			case 'h': help();
			          break;
			case 'e': printf("Showing/Hiding output from senzors...\n");
					  senzory=1-senzory;
					  break;
			case 'f': printf("Ready to go forward (actual speed: %d)... Use [+] and [-] to set speed\n", rychlost);
					  forward();
					  break;
            case 's': printf("Stoping... Use [f] or [c] to go forward (backward) \n");
			          stop(); set_leds(0x00);
					  cuvanie=0;
					  rychlost=0;
			          break;
            case 'c': printf("Getting back... Use [s] to stop...\n");
			          cuvanie=-1;  // cuvat budeme dokial nas niekto nezastavi
			          set_leds(0x09);
					  set_servo(1, SERVO_FW);
					  set_servo(4, SERVO_BW);
					  servo_123_on();
  					  servo_456_on();
			          break;
			case '+': 
					  if(rychlost!=5) rychlost++; 
					  set_servo(1, rychlost_kolesa('r', rychlost));
			  		  set_servo(4, rychlost_kolesa('l', rychlost));
			  		  printf("Setting new speed: %0d\n", rychlost);
			  		  break;
			case '-': 
					  if(rychlost!=1) rychlost--;
					  set_servo(1, rychlost_kolesa('r', rychlost));
					  set_servo(4, rychlost_kolesa('l', rychlost));
					  printf("Setting new speed: %0d\n", rychlost);
					  break;
            }
		};
		wait(10);
		
		timer++; timer%=10;  // casovac vypisov senzorov

		if(cuvanie>1) cuvanie--;   // ak mame spustene pocitadlo cuvania, tak ho znizme

		// ak je zapnute zobrazovanie senzorov, tak zobrazime (kazdych 50 ms)
		if(senzory==1){
		  if((timer==1)||(timer==6)){ 
		    printf("Senzors: %4d %4d\n", adc_vals[SENZOR1], adc_vals[SENZOR2]);
		  }
		}

		// ak sme prave skoncili cuvanie, nastavme kolesa spat na normalny chod
		if(cuvanie==1){
  		  cuvanie=0;
		  servo_123_on();
  		  servo_456_on();
	  	  set_servo(1, rychlost_kolesa('r', rychlost));
		  set_servo(4, rychlost_kolesa('l', rychlost));

		}

		// ak necuvame a mame nejaku rychlost nastavenu
		if((cuvanie==0)&&(rychlost!=0)){
		  
		  // ak je vozik otoceny jednym smerom, pridavame rychlost prvemu kolesu
		  if((adc_vals[SENZOR1]>100)&&(adc_vals[SENZOR2]<100)){ 
		  	if(smer!=-50-rychlost*5) smer--; set_leds(0x01);
		  }
		  
		  // ak je vozik otoceny druhym smerom, pridavame rychlost druhemu kolesu
		  if((adc_vals[SENZOR1]<100)&&(adc_vals[SENZOR2]>100)){ 
		    if(smer!=50+rychlost*5) smer++; set_leds(0x08);
		  }
		  
		  // ak je vozik uz spravne otoceny (rovno), tak spomalujeme alebo ideme rovno
		  if((adc_vals[SENZOR1]<100)&&(adc_vals[SENZOR2]<100)){ 
		    if(smer>0){ smer-=(krok*2); set_leds(0x04);if(smer<0) smer=0; }
		    if(smer<0){ smer+=(krok*2); set_leds(0x02); if(smer>0) smer=0; }
			if(smer==0) set_leds(0x00);
		  }
		  
		  // ak su oba senzory nad hranicu, tak je nejaky problem (moc otoceny vozik), takze cuvneme a pokracujeme
		  if((adc_vals[SENZOR1]>100)&&(adc_vals[SENZOR2]>100)){ 
		    cuvanie=150; cuvni();set_leds(0x06); continue;
		  }
	  	  
		  // nastavenie novej rychlosti oboch kolies
		  set_servo(1, rychlost_kolesa('r', rychlost));
		  set_servo(4, rychlost_kolesa('l', rychlost));

		  // vypisanie rychlosti kolies
		  //printf("Kolesa: %04d %04d (smer=%03d)\n",rychlost_kolesa('r', rychlost), rychlost_kolesa('l', rychlost), smer );
		}

	}

}


///////////////////////////////////////////////////////////////////////////////
